# $Id: cfilter.py,v 1.3 2001/01/23 11:44:34 petli Exp $
#
# cfilter.py -- Client filter functions
#
#    Copyright (C) 1999-2001  Peter Liljenberg <petli@ctrl-c.liu.se>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""plwm.filter contains various classes and functions which can be
used to create client filters.  These client filters can be used in
various places, e.g. for selecting whether a client window should have
a frame, which clients to cycle among for selection, etc.

A filter is called with one argument, a client object.  The return
value is true or false, depending on how the filter evaluated for the
client.

Simple filter functions:

  true		always true
  false		always false
  iconified     true if the client is iconified
  mapped        true if the client is mapped

Functions matching the client resource name (i.e. res_name or res_class):

  name(string)     true if the name exactly matches string
  re_name(regexp)  true if the name matches the regexp string
  glob_name(glob)  true if the name matches the glob string

Functions matching the client title:

  title(string)     true if the title exactly matches string
  re_title(regexp)  true if the title matches the regexp string
  glob_title(glob)  true if the title matches the glob string

Compound filters:

  And(filter, filter...)  true if all the filters are true
  Or(filter, filter...)   true if any of the filters are true
  Not(filter)             true if the filter is false

"""

import re
import fnmatch

# We must use class objects everywhere.  This is caused by the method
# semantics used in Python: anytime we get the value of an attribute
# in a class, and that value is a function, it is converted into a
# method object.  Since it is likely that filters will be assigned to
# class attributes, e.g. Client.start_iconified, we can't allow any
# filter to be ordinary functions.

# So we use Python objects with a __call__ method to circumvent this
# problem.  This gives us a performance penalty, but the alternative
# would be to write a recursive function which traverses the base
# classes, accessing __dict__ of each class to find the attribute we're
# searching.  This would avoid the casting of functions to methods, but
# could quite likely be just as ineffective as using objects.


class _True:
    def __call__(self, c):
	return 1
true = _True()
all = true

class _False:
    def __call__(self, c):
	return 0
false = _False()
none = false

class And:
    def __init__(self, *args):
	self.filters = args

    def __call__(self, c):
	for f in self.filters:
	    if not f(c):
		return 0
	return 1

class Or:
    def __init__(self, *args):
	self.filters = args

    def __call__(self, c):
	for f in self.filters:
	    if f(c):
		return 1
	return 0

class Not:
    def __init__(self, filter):
	self.filter = filter

    def __call__(self, c):
	return not self.filter(c)


class _Iconified:
    def __call__(self, c):
	return not c.is_mapped()
iconified = _Iconified()

class _Mapped:
    def __call__(self, c):
	return c.is_mapped()
mapped = _Mapped()


class _NameBase:
    def __init__(self, pattern):
	self.pattern = pattern

    def check(self, str):
	if str is None:
	    return self.pattern is None
	else:
	    return self.check_pattern(str)

class _StringName(_NameBase):
    def check_pattern(self, str):
	return self.pattern == str

class _ReName(_NameBase):
    def __init__(self, pattern):
	_NameBase.__init__(self, re.compile(pattern))

    def check_pattern(self, str):
	return self.pattern.search(str) is not None

class _GlobName(_NameBase):
    def check_pattern(self, str):
	return fnmatch.fnmatchcase(str, self.pattern)

class _Title:
    def __call__(self, c):
	return self.check(c.get_title())

class _Resource:
    def __call__(self, c):
	return self.check(c.res_name) or self.check(c.res_class)

class name(_StringName, _Resource): pass
class re_name(_ReName, _Resource): pass
class glob_name(_GlobName, _Resource): pass

class title(_StringName, _Title): pass
class re_title(_ReName, _Title): pass
class glob_title(_GlobName, _Title): pass
