# $Id: menu.py,v 1.8 2001/12/11 04:11:40 mwm Exp $
#
# menu.py -- Screen mixin to provide menus.
#
#    Copyright (C) 2001  Mike Meyer <mwm@mired.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


"menu - a mixin to provide menus for plwm screens."

import string
from Xlib import X, Xutil
from Xlib.XK import *
from keys import KeyGrabKeyboard


class MenuKeyHandler(KeyGrabKeyboard):
    """Template for handling a menu.

    MenuKeyHandler defines the following event handler methods:

    _up, _down - move the cursor up or down the menu.

    _do - perform the currently selected menu action.

    _abort - exit the menu with no actions.

    a-z and 0-9 are bound to _goto."""

    timeout = None

    def __init__(my, labels, window, gc, font, align):
        "Initialize the menu labels in window with gc and font."

        my.current = 0
        my.window = window
        my.gc = gc

        fontsize = font.query()
        my.high = (fontsize.font_ascent + fontsize.font_descent + 1)
        my.height = len(labels) * my.high

        width = 0
        entries = []
        for l in range(len(labels)):
            entry = Entry(labels[l], font, l * my.high + fontsize.font_ascent + 1)
            w = entry.width
            if w > width: width = w
            entries.append(entry)
        my.width = width + 4

        for i in range(len(entries)):
            entries[i].setup(window, gc, my.width, align, i)
        my.entries = entries

    def _start(my, wm, time, action):
        "Start the menu by initializing everything appropriately."

        my.action = action
        wm.dispatch.add_handler(X.Expose, my._redraw)
        KeyGrabKeyboard.__init__(my, wm, time)

    def _redraw(my, event = None):
        "Redraw the window, with highlights"

        my.window.clear_area(width = my.width, height = my.height)
        for i in range(len(my.entries)):
            my.entries[i].redraw()
        my.window.fill_rectangle(my.gc, 0, my.current * my.high, my.width, my.high)

    def _up(my, event):
        "Move the menu selection up."

        my.current = my.current - 1
        if my.current < 0: my.current = len(my.entries) - 1
        my._redraw()

    def _down(my, event):
        "Move the menu selection down"

        my.current = my.current + 1
        if my.current >= len(my.entries): my.current = 0
        my._redraw()
        
    def _do(my, event):
        "Run it!"
        my.window.unmap()
        my.action(my.entries[my.current].name)
        my._cleanup()

    def _abort(my, event):
        my.window.unmap()
        my._cleanup()


class MenuCharHandler(MenuKeyHandler):
    """MenuCharHandler allows "one-key access" to menu entries.

    This adds the _goto method, which goes to the first menu label
    that starts with a character >= the event keycode, then binds
    a-z and 0-9 to that method."""

    def _goto(my, event):
        "Move to the first entry with a label that starts >= char"

        char = chr(my.wm.display.keycode_to_keysym(event.detail, 0))
        length = len(my.entries)
        for i in range(length):
            if char <= my.entries[i].name[0].lower():
                break
        if i < length: my.current = i
        else: my.current = length - 1
        my._redraw()

for c in string.digits + string.lowercase:
    setattr(MenuCharHandler, c, MenuCharHandler._goto)


class screenMenu:
    """PLWM Screen mixin to provide a per-screen menu.

    This mixin requires the color and font mixins be in the screen class."""

    menu_fontname = "9x15bold"
    menu_foreground = "black"
    menu_background = "white"
    menu_borderwidth = 3
    menu_handler = MenuKeyHandler

    def __screen_client_init__(my):
        "Create the initial window, and tag it as internal."

        fg = my.get_color(my.menu_foreground)
        bg = my.get_color(my.menu_background)
        my.menu_font = my.wm.get_font(my.menu_fontname, 'fixed')

        my.menu_window = \
                       my.root.create_window(1, 1, 1, 1, my.menu_borderwidth,
                                             X.CopyFromParent, X.InputOutput,
                                             X.CopyFromParent,
                                             background_pixel = bg,
                                             border_pixel = fg,
                                             event_mask = (X.KeyPressMask |
                                                           X.StructureNotifyMask |
                                                           X.ExposureMask))

        my.menu_gc = my.menu_window.create_gc(font = my.menu_font,
                                              function = X.GXinvert,
                                              foreground = fg, background = bg)
        my.add_internal_window(my.menu_window, None)

    def menu_make(my, labels, align = 'center'):
        """Create a menu of labels.

        Returns the width and height of the resulting menu."""

        my.menu_menu = my.menu_handler(labels, my.menu_window,
                                       my.menu_gc, my.menu_font, align)
        width = my.menu_menu.width
        height = my.menu_menu.height
        return width, height

    def menu_run(my, x, y, action):
        "Instantiate the menu, and return the label or None."


        if x < 0: x = 0
        if x + my.menu_menu.width > my.root_width:
            x = my.root_width - my.menu_menu.width
        if y < 0: y = 0
        if y + my.menu_menu.height > my.root_height:
            y = my.root_height - my.menu_menu.height
        my.menu_window.configure(x = x, y = y, width = my.menu_menu.width,
                                 height = my.menu_menu.height,
                                 stack_mode = X.Above)
        my.menu_window.map()
        my.wm.display.set_input_focus(my.menu_window, X.RevertToPointerRoot,
                                      X.CurrentTime)
        my.menu_menu._start(my.wm, X.CurrentTime, action)
                                     

class Entry:
    "A class for entries in a pointerless menu."

    def __init__(my, label, font, where):
        "Figure out where to draw this string."

        my.name = label
        ext = font.query_text_extents(label)
        my.width = ext.overall_width
        my.y = where

    def setup(my, window, gc, width, align, count):
        "Save the drawing position."

        my.window, my.gc = window, gc
        if align == 'left':
            my.x = 0
        elif align == 'center':
            my.x = (width - my.width) / 2
        else:	# right
            my.x = width - my.width

    def redraw(my):
        "Draw myself."

        my.window.image_text(my.gc, my.x, my.y, my.name)
